﻿using System.Windows.Forms;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 조부모 그리드 뷰를 설정한다.

            GridView grandParentGridView = new GridView();

            grandParentGridView.Name = "grandParentGridView";

            grandParentGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            grandParentGridView.OptionsDetail.EnableMasterViewMode = true;

            grandParentGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 부모 그리드 뷰를 설정한다.

            GridView parentGridView = new GridView();

            parentGridView.Name = "parentGridView";

            parentGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            parentGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 자식 그리드 뷰를 설정한다.

            GridView childGridView = new GridView();

            childGridView.Name = "childGridView";

            childGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            childGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 그리드 레벨 노드 2를 설정한다.

            GridLevelNode gridLevelNode2 = new GridLevelNode();

            gridLevelNode2.RelationName  = "Relation2";
            gridLevelNode2.LevelTemplate = childGridView;

            #endregion
            #region 그리드 레벨 노드 1을 설정한다.

            GridLevelNode gridLevelNode1 = new GridLevelNode();

            gridLevelNode1.RelationName  = "Relation1";
            gridLevelNode1.LevelTemplate = parentGridView;

            gridLevelNode1.Nodes.Add(gridLevelNode2);

            #endregion
            #region 그리드 컨트롤을 설정한다.

            GridControl gridControl = new GridControl();

            gridControl.Name     = "gridControl";
            gridControl.Dock     = DockStyle.Fill;
            gridControl.MainView = grandParentGridView;

            gridControl.ViewCollection.AddRange(new BaseView[] { grandParentGridView, parentGridView, childGridView });

            gridControl.LevelTree.Nodes.Add(gridLevelNode1);

            Controls.Add(gridControl);

            gridControl.DataSource = Customer.GetCustomerList();

            #endregion

            grandParentGridView.BestFitColumns();

            grandParentGridView.MasterRowExpanded += gridView_MasterRowExpanded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 그리드 뷰 마스터 행 확장시 처리하기 - gridView_MasterRowExpanded(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 확장시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowExpanded(object sender, CustomMasterRowEventArgs e)
        {
            GridView gridView = sender as GridView;

            GridView childGridView = gridView.GetDetailView(e.RowHandle, e.RelationIndex) as GridView;

            if(childGridView.Name == "grandParentGridView")
            {
                childGridView.Name = "parentGridView";

                childGridView.BestFitColumns();

                childGridView.MasterRowExpanded += gridView_MasterRowExpanded;
            }
            else if(childGridView.Name == "parentGridView")
            {
                childGridView.Name = "childGridView";

                childGridView.BestFitColumns();
            }
        }

        #endregion
    }
}